#! /bin/sh
#
# Starts pcmcia cardmanaging services...
#
#
# $1 : Modus operandi {start|stop|restart}
# ret: {0:process all ok | 1:part/whole process failed}
#
##
#$ Thursday 02/02/06 15:00:16 md(root) mathieu.deschamps@com2gether.net$
##
INTFILE=/etc/network/interfaces
NETSERV=/etc/network/interfaces.service
PCMCIAOPTION=/etc/pcmcia/pcmcia
PCMCIAMODULES=/lib/modules/`uname -r`/pcmcia
RUN=/var/lib/pcmcia
MODLIST="a)pcmcia_core b)i82365 c)ds "
##
#
#
MODUS=$1
# Maybe its run as a non rcS-launcher call (ie manual call)
[ "x$VERBOSE" = "x" ] && VERBOSE=no
[ "x$2" != "x" ] && SERVICE="$2"
#
#
# Source PCMCIA configuration.
if [ -f $PCMCIAOPTION ] ; then
    . $PCMCIAOPTION
else
     echo  "Starting pcmcia... Missing $PCMCIAOPTION configuration option file !"
     exit 1
fi
#
#
#
cleanup()
{
    while read SN CLASS MOD INST DEV EXTRA ; do
        if [ "$SN" != "Socket" ] ; then
            /etc/pcmcia/$CLASS stop $DEV 2> /dev/null
        fi
    done
}

#
# Module handling procedure
# $1 {IN (inserting) | OUT (removing)}
# $2 MODLIST (typo "a)module1 b)module2 c)...")
# RET {0 | 1 }
#
modules ()
{
	MODE=onerror
	[ "x$1" = "xIN" -o "x$1" = "xin" ] && MODE= && PH=inserting
	[ "x$1" = "xOUT" -o "x$1" = "xout" ] && MODE=-r && PH=removing
	[ "$MODE" = "onerror" ] && echo "modules() mode is {in | out} !" && return 1
	if [ "x$2" = "x" ] ; then 
		echo "modules() modlist is void !"	
		return 1
	else
		MODLIST="$2"
	fi
	RET=0

	for module in `echo -e "$MODLIST" | tr ' ' '\n' | sort $MODE | cut -f2 -d")"`
	do
		[ $VERBOSE = yes ] && OPTION="pc_debug=2" || OPTION=

		if [ "x$MODE" = "x" ] ; then
                        [ "x$module" = "xpcmcia_core" ] && OPTION="$OPTION $CORE_OPTS"
			[ "x$module" = "xi82365" ] && OPTION="$OPTION $PCIC_OPTS"
		fi

                [ $VERBOSE = yes ]                      &&\
	                echo -en "\t$PH $module ${OPTION} "

                if [ $VERBOSE = yes ] ; then
	                /sbin/modprobe $MODE $module ${OPTION}  >/dev/null
		else
			/sbin/modprobe $MODE $module ${OPTION} 	>/dev/null 2>&1
		fi
                RET=$?
                if [ $RET -ne 0 ] ; then
			[ $VERBOSE = yes ] 			&&\       
	                        echo -e "\t error on $PH $module ${OPTION} "
		else
			[ $VERBOSE = yes ] 			&&\
	                        echo -en " ok "
                fi
		sleep 1
	done
	return $RET
}

start () {
	echo -n "Starting pcmcia..."

	if [ -L $RUN/scheme -o ! -O $RUN/scheme ] ; then rm -f $RUN/scheme ; fi
        if [ ! -f $RUN/sheme ] ; then umask 022 ; touch $RUN/scheme ; fi
        if [ "$SCHEME" ] ; then umask 022 ; echo $SCHEME > $RUN/scheme ; fi

	# Check if modules has been already inserted
	if ! grep -q pcmcia /proc/devices ; then	
		[ $VERBOSE = yes ] &&  echo -en "\t Pcmcia module will be installed..."
		modules in "$MODLIST"
	else
		[ $VERBOSE = yes ] &&  echo -en "\t Module present, no insertion required."	
	fi
	
	# Check if cardmgr is already running
        if [ -s /var/run/cardmgr.pid ] && kill -0 `cat /var/run/cardmgr.pid` 2>/dev/null ; then
            echo -en "\t cardmgr is already running."
        else
            if [ -r $RUN/stab ] ; then
                cat $RUN/stab | cleanup
            fi
		# Start cardmgr
            	if [ $VERBOSE = yes ] ; then
			/sbin/cardmgr -v $CARDMGR_OPTS 
		else
			/sbin/cardmgr -q $CARDMGR_OPTS >/dev/null 2>&1	
		fi
        fi

	touch /var/lock/subsys/pcmcia 2>/dev/null

	echo "done."
}
stop () {
	echo -n "Stopping pcmcia..."

	cardctl eject 2>&1 >/dev/null 
	sleep 1
	if [ -s /var/run/cardmgr.pid ] ; then
            kill -9 `cat /var/run/cardmgr.pid`
            # Give cardmgr a few seconds to handle the signal
            while [ 1 ]; do
                kill -0 $PID 2>/dev/null || break
                sleep 2
            done
        fi

        if  grep -q pcmcia /proc/devices ; then
		modules out "$MODLIST"
	else
		[ $VERBOSE = yes ] &&  echo -en "\t No module present, no removal required."
        fi
        rm -f /var/lock/subsys/pcmcia

	echo "done."
}
restart () {
	stop 
	start
}

status () {
        pid=`/bin/pidof cardmgr`
        if [ "$pid" != "" ] ; then
            echo "cardmgr (pid $pid) is running..."
        else
            echo "cardmgr is stopped"
        fi
}

case "$1" in
	start|"")
		start
		;;
	stop)
		stop
		;;
	restart)
		restart
		;;
	status)
		status
		;;
	reload)
		echo "Reloading $DESC configuration files."
        	kill -1 `cat /var/run/cardmgr.pid` 2>/dev/null
		;;
	*)
		echo $"Usage: $0 {start|stop|status|restart|reload}"
	        exit 1
		;;
esac

exit $?
